#ifndef __MMPLAYER_H
#define __MMPLAYER_H

#define STRICT
#include <string.h>
#include <windows.h>
#include <mmsystem.h>
#include <mem.h>
#pragma hdrstop

#define MS MCI_FORMAT_MILLISECONDS
#define HMS MCI_FORMAT_HMS
#define MSF MCI_FORMAT_MSF
#define FRAMES MCI_FORMAT_FRAMES
#define BYTES MCI_FORMAT_BYTES
#define SAMPLES MCI_FORMAT_SAMPLES
#define TMSF MCI_FORMAT_TMSF
#define CHAPTERS MCI_FORMAT_CHAPTERS
#define MM_SYNC MCI_NOTIFY | MCI_WAIT
#define MM_ASYNC MCI_NOTIFY

class TMMPlayer {
	public:
		TMMPlayer(HWND hWnd);
		TMMPlayer(HWND hWnd, const char* device);
		TMMPlayer(HWND hWnd, const char* device,
							const char* filename, DWORD flags = 0);
		~TMMPlayer();
		DWORD Open(const char* filename, const char* device = 0);
		DWORD Open(const char* filename,
								const char* device,
								HWND parent,
								DWORD style,
								DWORD flags);
		DWORD Close();
		DWORD Play(DWORD flags = MM_SYNC);
		DWORD Play(const char* filename,
								DWORD flags = MM_SYNC);
		DWORD Play(DWORD from, DWORD to,
								DWORD flags = MM_SYNC);
		DWORD Record(DWORD flags = MCI_NOTIFY | MCI_RECORD_OVERWRITE);
		DWORD Record(DWORD from, DWORD to,
								DWORD flags = MM_SYNC | MCI_RECORD_OVERWRITE);
		DWORD Save(const char* filename = 0,
								DWORD flags = MCI_SAVE_FILE | MM_SYNC);
		DWORD Load(const char* filename, DWORD flags = MCI_LOAD_FILE | MM_SYNC);
		DWORD Delete(DWORD from = 0, DWORD to = 0);
		DWORD Stop();
		DWORD Seek(DWORD to = 0, DWORD flags = MCI_SEEK_TO_START);
		DWORD Pause(DWORD flags = MM_SYNC);
		DWORD Resume();

		DWORD Erase() {return Delete(0, GetLength());}
		DWORD Rewind() {return Seek();}
		DWORD FastForward() {return Seek(0, MM_SYNC | MCI_SEEK_TO_END);}

		BOOL	CanEject() {return (BOOL)GetDeviceCaps(MCI_GETDEVCAPS_CAN_EJECT);}
		BOOL	CanPlay() {return (BOOL)GetDeviceCaps(MCI_GETDEVCAPS_CAN_PLAY);}
		BOOL	CanRecord() {return (BOOL)GetDeviceCaps(MCI_GETDEVCAPS_CAN_RECORD);}
		BOOL	CanSave() {return (BOOL)GetDeviceCaps(MCI_GETDEVCAPS_CAN_SAVE);}
		BOOL	HasAudio() {return (BOOL)GetDeviceCaps(MCI_GETDEVCAPS_HAS_AUDIO);}
		BOOL	HasVideo() {return (BOOL)GetDeviceCaps(MCI_GETDEVCAPS_HAS_VIDEO);}
		DWORD IsMediaPresent() {return (BOOL)GetStatusItem(MCI_STATUS_MEDIA_PRESENT);}
		BOOL	IsOpen() {return DeviceOpen;}
		BOOL	IsReady() {return (BOOL)GetStatusItem(MCI_STATUS_READY);}
		BOOL	UsesFiles() {return (BOOL)GetDeviceCaps(MCI_GETDEVCAPS_USES_FILES);}

		DWORD GetStatusItem(DWORD item);
		DWORD GetDeviceCaps(DWORD item);
		DWORD GetSysInfoItem(DWORD item, char* info, int dev);
		DWORD GetInfoItem(DWORD item, char* info);
		DWORD GetDeviceName(char* name);
		DWORD GetDeviceType() {return GetDeviceCaps(MCI_GETDEVCAPS_DEVICE_TYPE);}
		DWORD GetFileName(char* info) {return GetInfoItem(MCI_INFO_FILE, info);}
		DWORD GetPosition()  {return GetStatusItem(MCI_STATUS_POSITION);}
		DWORD GetLength() {return GetStatusItem(MCI_STATUS_LENGTH);}
		DWORD GetMode() {return GetStatusItem(MCI_STATUS_MODE);}
		DWORD GetTimeFormat() {return GetStatusItem(MCI_STATUS_TIME_FORMAT);}
		DWORD GetVolume();
		DWORD	GetStatus() {return GetStatusItem(MCI_STATUS_MODE);}

		DWORD Cut(DWORD flags = MM_ASYNC);
		DWORD Paste(DWORD flags = MM_ASYNC);
		DWORD Copy(DWORD flags = MM_ASYNC);
		DWORD Set(DWORD flags, DWORD options = 0);
		DWORD SetAudioOff() {return Set(MCI_SET_AUDIO |
								  MCI_SET_OFF, MCI_SET_AUDIO_ALL);}
		DWORD SetAudioOn() {return Set(MCI_SET_AUDIO |
								  MCI_SET_ON, MCI_SET_AUDIO_ALL);}
		void	SetParent(HWND hWnd) {hWindow = hWnd;}
		DWORD SetTimeFormat(DWORD format) {return Set(MCI_SET_TIME_FORMAT, format);}
		DWORD SetVideoOff() {return Set(MCI_SET_VIDEO | MCI_SET_OFF);}
		DWORD SetVideoOn() {return Set(MCI_SET_VIDEO | MCI_SET_ON);}
		DWORD SetPosition(DWORD position) {return Seek(position);}
		DWORD SetVolume(int volume);
		DWORD SetVolume(int lvol, int rvol);
		void	SetErrorReporting(BOOL err) {ErrorReporting = err;}
		DWORD ErrCode;
	private:
		void ReportError(const char* title = "MultiMedia Error");
		HWND hWindow;
		UINT DeviceID;
		BOOL DeviceOpen;
		UINT ErrorReporting;
};

#endif